/* Yet Another Forum.NET
 * Copyright (C) 2003-2005 Bjrnar Henden
 * Copyright (C) 2006-2009 Jaben Cargman
 * http://www.yetanotherforum.net/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
using System;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using YAF.Classes.Core;

namespace YAF.Classes.UI
{
  /// <summary>
  /// Summary description for SmartScroller.
  /// </summary>
  public class SmartScroller : Control
  {
    /* Ederon : 6/16/2007 - conventions */

    /// <summary>
    /// The _hid scroll left.
    /// </summary>
    private HtmlInputHidden _hidScrollLeft = new HtmlInputHidden();

    /// <summary>
    /// The _hid scroll top.
    /// </summary>
    private HtmlInputHidden _hidScrollTop = new HtmlInputHidden();

    /// <summary>
    /// The _the form.
    /// </summary>
    private HtmlForm _theForm = null;

    /// <summary>
    /// Initializes a new instance of the <see cref="SmartScroller"/> class.
    /// </summary>
    public SmartScroller()
    {
    }

    /// <summary>
    /// The on init.
    /// </summary>
    /// <param name="e">
    /// The e.
    /// </param>
    protected override void OnInit(EventArgs e)
    {
      this._hidScrollLeft.ID = "scrollLeft";
      this._hidScrollTop.ID = "scrollTop";

      Controls.Add(this._hidScrollLeft);
      Controls.Add(this._hidScrollTop);

      string scriptString =
        @"
  function yaf_SmartScroller_GetCoords()
  {
    var scrollX, scrollY;
    if (document.all)
    {
      if (!document.documentElement.scrollLeft)
        scrollX = document.body.scrollLeft;
      else
        scrollX = document.documentElement.scrollLeft;

      if (!document.documentElement.scrollTop)
        scrollY = document.body.scrollTop;
      else
        scrollY = document.documentElement.scrollTop;
    }
    else
    {
      scrollX = window.pageXOffset;
      scrollY = window.pageYOffset;
    }
	  jQuery('#" +
        this._hidScrollLeft.ClientID + @"').val( scrollX );
		jQuery('#" + this._hidScrollTop.ClientID +
        @"').val( scrollY );
  }

  function yaf_SmartScroller_Scroll()
  {
		var x = jQuery('#" + this._hidScrollLeft.ClientID +
        @"').val();
		var y = jQuery('#" + this._hidScrollTop.ClientID +
        @"').val();
		if (x || y) window.scrollTo(x,y);
  }

	function yaf_SmartScroller_Reset()
	{
	  jQuery('#" + this._hidScrollLeft.ClientID +
        @"').val( 0 );
		jQuery('#" + this._hidScrollTop.ClientID +
        @"').val( 0 );	
		// force change...
		window.scrollTo(0,0);
	}

	jQuery(window).bind('scroll', yaf_SmartScroller_GetCoords);
	jQuery(window).bind('click', yaf_SmartScroller_GetCoords);
	jQuery(window).bind('keypress', yaf_SmartScroller_GetCoords);
	jQuery(document).ready(yaf_SmartScroller_Scroll);
";

      YafContext.Current.PageElements.RegisterJQuery();
      YafContext.Current.PageElements.RegisterJsBlock("SmartScrollerJs", scriptString);
    }

    /// <summary>
    /// The render.
    /// </summary>
    /// <param name="writer">
    /// The writer.
    /// </param>
    protected override void Render(HtmlTextWriter writer)
    {
      Page.VerifyRenderingInServerForm(this);
      base.Render(writer);
    }

    /// <summary>
    /// The register startup reset.
    /// </summary>
    public void RegisterStartupReset()
    {
      Reset();
      const string script = @"Sys.WebForms.PageRequestManager.getInstance().add_endRequest(yaf_SmartScroller_Reset);";
      YafContext.Current.PageElements.RegisterJsBlockStartup("SmartScrollerResetJs", script);
    }

    /// <summary>
    /// The reset.
    /// </summary>
    public void Reset()
    {
      this._hidScrollLeft.Value = "0";
      this._hidScrollTop.Value = "0";
    }
  }
}