/* Yet Another Forum.NET
 * Copyright (C) 2006-2009 Jaben Cargman
 * http://www.yetanotherforum.net/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
using System;
using System.Web.UI;
using YAF.Classes.UI;

namespace YAF.Controls
{
  /// <summary>
  /// Makes a very simple localized label
  /// </summary>
  public class LocalizedLabel : BaseControl
  {
    /// <summary>
    /// The _enable bb code.
    /// </summary>
    protected bool _enableBBCode = false;

    /// <summary>
    /// The _localized page.
    /// </summary>
    protected string _localizedPage = string.Empty;

    /// <summary>
    /// The _localized tag.
    /// </summary>
    protected string _localizedTag = string.Empty;

    /// <summary>
    /// The _param 0.
    /// </summary>
    protected string _param0 = string.Empty;

    /// <summary>
    /// The _param 1.
    /// </summary>
    protected string _param1 = string.Empty;

    /// <summary>
    /// The _param 2.
    /// </summary>
    protected string _param2 = string.Empty;

    /// <summary>
    /// Initializes a new instance of the <see cref="LocalizedLabel"/> class.
    /// </summary>
    public LocalizedLabel()
      : base()
    {
    }

    /// <summary>
    /// Gets or sets LocalizedPage.
    /// </summary>
    public string LocalizedPage
    {
      get
      {
        return this._localizedPage;
      }

      set
      {
        this._localizedPage = value;
      }
    }

    /// <summary>
    /// Gets or sets LocalizedTag.
    /// </summary>
    public string LocalizedTag
    {
      get
      {
        return this._localizedTag;
      }

      set
      {
        this._localizedTag = value;
      }
    }

    /// <summary>
    /// Gets or sets Param0.
    /// </summary>
    public string Param0
    {
      get
      {
        return this._param0;
      }

      set
      {
        this._param0 = value;
      }
    }

    /// <summary>
    /// Gets or sets Param1.
    /// </summary>
    public string Param1
    {
      get
      {
        return this._param1;
      }

      set
      {
        this._param1 = value;
      }
    }

    /// <summary>
    /// Gets or sets Param2.
    /// </summary>
    public string Param2
    {
      get
      {
        return this._param2;
      }

      set
      {
        this._param2 = value;
      }
    }

    /// <summary>
    /// Gets or sets a value indicating whether EnableBBCode.
    /// </summary>
    public bool EnableBBCode
    {
      get
      {
        return this._enableBBCode;
      }

      set
      {
        this._enableBBCode = value;
      }
    }

    /// <summary>
    /// Shows the localized text string (if available)
    /// </summary>
    /// <param name="output">
    /// </param>
    protected override void Render(HtmlTextWriter output)
    {
      string localizedItem = GetCurrentItem();

      // convert from YafBBCode to HTML
      if (this._enableBBCode)
      {
        localizedItem = YafBBCode.MakeHtml(localizedItem, true, false);
      }

      output.BeginRender();
      output.Write(String.Format(localizedItem, this._param0, this._param1, this._param2));
      output.EndRender();
    }

    /// <summary>
    /// The get current item.
    /// </summary>
    /// <returns>
    /// The get current item.
    /// </returns>
    protected string GetCurrentItem()
    {
      if (Site != null && Site.DesignMode == true)
      {
        return String.Format("[PAGE:{0}|TAG:{1}]", LocalizedPage, LocalizedTag);
      }
      else if (!String.IsNullOrEmpty(this._localizedPage) && !String.IsNullOrEmpty(this._localizedTag))
      {
        return PageContext.Localization.GetText(LocalizedPage, LocalizedTag);
      }
      else if (!String.IsNullOrEmpty(this._localizedTag))
      {
        return PageContext.Localization.GetText(LocalizedTag);
      }

      return null;
    }
  }
}